##### FASS_Descriptive_SeverityStudy.R
#####

#####   ScoreDescriptive
###     
##       

ScoreDescriptive <- function (ScoreData, PopulationSize = NULL )
{
 
  #####   oFASS DESCRIPTIVE
  ###     Count descriptive for oFASS
  oFASS_Levels <- levels(ScoreData[, oFASS_Var])
  
  if (is.null(oFASS_Levels))
  {
    oFASS_Levels <- c(1,2,3,4,5)
  }
  
  
  oFASS_Counts <- CountDescriptive( DataSet = ScoreData, 
                                    Variable = oFASS_Var, 
                                    Values = oFASS_Levels,
                                    PopulationSize = PopulationSize)
  
  #####   oFASS3 DESCRIPTIVE
  ###     Count descriptive for oFASS
  oFASS3_Levels <- levels(ScoreData[, oFASS3_Var])
  
  if (is.null(oFASS3_Levels))
  {
    oFASS3_Levels <- c(1,2,3)
  }
  
  
  oFASS3_Counts <- CountDescriptive( DataSet = ScoreData, 
                                     Variable = oFASS3_Var, 
                                     Values = oFASS3_Levels,
                                     PopulationSize = PopulationSize)
  
  
  ##### nFASS DESCRIPTIVE
  ###   Statistic parameters of the population after computing the nFASS.
  #     - Statistics: Average, median, range, IQ range.
  #     - Classification: By grades
  
  nFASS_Stat <- vector(mode = "list", length = 5)
  GradeData <- ScoreData[which(ScoreData[ , oFASS_Var] == oFASS_Levels[1]), ] 
  TableScoreDesc <- c(round(summary(  as.numeric(unlist(GradeData[ ,nFASS_Var]))), 2),round(sd(as.numeric(unlist(GradeData[ ,nFASS_Var])))))

  for (Level in oFASS_Levels[2:length(oFASS_Levels)])
  {
    GradeData <- ScoreData[which(ScoreData[ , oFASS_Var] == Level), ]  
    TableScoreDesc <- rbind(TableScoreDesc, c(round(summary(as.numeric(unlist(GradeData[ ,nFASS_Var]))), 2),round(sd(as.numeric(unlist(GradeData[ ,nFASS_Var]))),2)))
  }
  
  colnames(TableScoreDesc) <- c("Min.", "1st Qu.", "Median","Mean","3rd Qu.","Max.","SD")    

  rownames(TableScoreDesc) <- paste0("Grade_", oFASS_Levels)
  
  #####   PRINTING RESULTS
  ###     (1) oFASS descriptive
  ##      (2) nFASS descriptive
  
  subDir<-"Descriptive_FASS"
  mainDir<-ProjectLocation
  
  #create the folder of the results
  ifelse(!dir.exists(file.path(mainDir, subDir)), dir.create(file.path(mainDir, subDir)), FALSE)
  Filepath <- paste0(file.path(mainDir, subDir),"/")
  
  ### (1)
  Filelocalname <- paste0("ScoreDescriptive_", oFASS_Var, ".txt")
  sink( paste0(Filepath, Filelocalname) ) 
  print("----- oFASS descriptive -----")
  print(oFASS_Counts)
  writeLines("\n")
  print("----- oFASS3 descriptive -----")
  print(oFASS3_Counts)
  sink()
  
  ### (2)
  Filelocalname <- paste0("ScoreDescriptive_", nFASS_Var, ".txt")
  sink( paste0(Filepath, Filelocalname) ) 
  print("----- nFASS descriptive -----")
  print(TableScoreDesc)
  sink()
  
}

CountDescriptive <- function (DataSet, Variable, Values = NULL, PopulationSize = NULL, NoDuplicates = FALSE)
{
  
  print(paste0("Counting members of ", Variable))
  
  if (is.null(Variable))
  {
    return(NULL)
  }
  
  
  #####   COUNT THE NUMBER OF APPEARANCES OF EACH VALUE IN LIST
  ###       (1) Count by value and store it in the given vector.
  ##        (2) If more rows than elements were counted, classify the rest in "Other".
  ##        (3) Compute relative values in a new column. Optionally
  ##          (b) Optionally, compute also with respect to population (instead of food forms) size
  ##        (4) Compute total values and add the name to names list.
  ##        (5) Bind by column.
  
  
  if (is.null(Values))
  {
    Values <- levels(factor(DataSet[, Variable]))
  }
  
  if (NoDuplicates)
  {
    DataSetAux <- DataSet[ , c(PatientId_Var, Variable)]
    DataSetAux <- DataSetAux[!duplicated(DataSetAux), ]
  } else {DataSetAux <- DataSet[ , c(PatientId_Var, Variable)]}
  
  ###   (1)
  Counts <- vector (length = length(Values))
  Index <- 1
  for (Value in Values)
  {
    Counts[Index] <- nrow(DataSetAux[which(DataSetAux[, Variable] == Value),])
    Index <- Index+1
  }
  
  ###   (2)
  Values_Ext <- Values
  if (nrow(DataSetAux) > sum(Counts))
  {
    ##  Count the number of "others"
    N_Others <- nrow(DataSetAux) - sum(Counts)
    Counts <- c(Counts, N_Others)
    
    ##  Add "Others" to names list
    Values_Ext <- c(Values_Ext, "Others: ")
  }
  
  ###   (3)
  Counts_Rel <- Counts/sum(Counts)
  Counts_Rel <- c(Counts_Rel, sum(Counts_Rel))
  
  ### (3b)
  if (!is.null(PopulationSize))
  {
    Counts_Rel_Pop <- Counts/PopulationSize
    Counts_Rel_Pop <- c(Counts_Rel_Pop, PopulationSize)
  }
  
  ###   (4)
  Counts <- c(Counts, sum(Counts))
  Values_Ext <- c(Values_Ext, "Total: ")
  
  ###   (5)
  if (!is.null(PopulationSize))
  {
    VarCDesc <- cbind(Values_Ext, as.numeric(Counts), round(Counts_Rel,5 ), round(Counts_Rel_Pop, 5))
  } else {
    VarCDesc <- cbind(Values_Ext, as.numeric(Counts), round(Counts_Rel,5 ))
  }
  
  return(VarCDesc)
  
}
